#include "stdafx.h"
#include "ChannelList.h"




CChannelInfo::CChannelInfo(int Space,int Channel,int Index,int No,LPCTSTR pszName)
{
	m_Space=Space;
	m_Channel=Channel;
	m_ChannelIndex=Index;
	m_ChannelNo=No;
	lstrcpyn(m_szName,pszName,MAX_CHANNEL_NAME);
}


CChannelInfo::CChannelInfo(const CChannelInfo &Info)
{
	m_Space=Info.m_Space;
	m_Channel=Info.m_Channel;
	m_ChannelIndex=Info.m_ChannelIndex;
	m_ChannelNo=Info.m_ChannelNo;
	lstrcpy(m_szName,Info.m_szName);
}




CChannelList::CChannelList()
{
	m_NumChannels=0;
	m_ppList=NULL;
	m_ListLength=0;
}


CChannelList::~CChannelList()
{
	Clear();
}


bool CChannelList::AddChannel(int Space,int Channel,int Index,int No,LPCTSTR pszName)
{
	CChannelInfo Info(Space,Channel,Index,No,pszName);

	return AddChannel(Info);
}


bool CChannelList::AddChannel(const CChannelInfo &Info)
{
	if (m_NumChannels==m_ListLength) {
		if (m_ListLength==0)
			m_ListLength=16;
		else
			m_ListLength*=2;
		m_ppList=(CChannelInfo**)realloc(m_ppList,
										m_ListLength*sizeof(CChannelInfo*));
	}
	m_ppList[m_NumChannels++]=new CChannelInfo(Info);
	return true;
}


const CChannelInfo *CChannelList::GetChannelInfo(int No) const
{
	if (No<0 || No>=m_NumChannels)
		return NULL;
	return m_ppList[No];
}


int CChannelList::GetSpace(int No) const
{
	if (No<0 || No>=m_NumChannels)
		return -1;
	return m_ppList[No]->GetSpace();
}


int CChannelList::GetChannel(int No) const
{
	if (No<0 || No>=m_NumChannels)
		return -1;
	return m_ppList[No]->GetChannel();
}


int CChannelList::GetChannelNo(int No) const
{
	if (No<0 || No>=m_NumChannels)
		return -1;
	return m_ppList[No]->GetChannelNo();
}


int CChannelList::GetChannelIndex(int No) const
{
	int Index;

	if (No<0 || No>=m_NumChannels)
		return -1;
	Index=m_ppList[No]->GetChannelIndex();
	return Index;
}


LPCTSTR CChannelList::GetName(int No) const
{
	if (No<0 || No>=m_NumChannels)
		return TEXT("");
	return m_ppList[No]->GetName();
}


void CChannelList::Clear()
{
	if (m_NumChannels>0) {
		int i;

		for (i=m_NumChannels-1;i>=0;i--)
			delete m_ppList[i];
		free(m_ppList);
		m_NumChannels=0;
		m_ppList=NULL;
		m_ListLength=0;
	}
}


int CChannelList::FindChannelNo(int No) const
{
	int i;

	for (i=0;i<m_NumChannels;i++) {
		if (GetChannelNo(i)==No)
			return i;
	}
	return -1;
}


int CChannelList::GetNextChannelNo(int ChannelNo,bool fWrap) const
{
	int i;
	int Channel,Min,No;

	Channel=1000;
	Min=1000;
	for (i=0;i<m_NumChannels;i++) {
		No=GetChannelNo(i);
		if (No>ChannelNo && No<Channel)
			Channel=No;
		if (No<Min)
			Min=No;
	}
	if (Channel==1000) {
		if (fWrap)
			return Min;
		return 0;
	}
	return Channel;
}


int CChannelList::GetPrevChannelNo(int ChannelNo,bool fWrap) const
{
	int i;
	int Channel,Max,No;

	Channel=0;
	Max=0;
	for (i=0;i<m_NumChannels;i++) {
		No=GetChannelNo(i);
		if (No<ChannelNo && No>Channel)
			Channel=No;
		if (No>Max)
			Max=No;
	}
	if (Channel==0) {
		if (fWrap)
			return Max;
		return 0;
	}
	return Channel;
}


int CChannelList::GetMaxChannelNo() const
{
	int Max,No;
	int i;

	Max=0;
	for (i=0;i<m_NumChannels;i++) {
		No=GetChannelNo(i);
		if (No>Max)
			Max=No;
	}
	return Max;
}




CTuningSpaceList::CTuningSpaceList()
{
	m_ppChannelList=NULL;
	m_NumSpaces=0;
}


CTuningSpaceList::~CTuningSpaceList()
{
	Clear();
}


CChannelList *CTuningSpaceList::GetChannelList(int Space)
{
	if (Space<0 || Space>=m_NumSpaces) {
		TRACE(TEXT("CTuningSpaceList::GetChannelList() : Out of range %d\n"),Space);
		return NULL;
	}
	return m_ppChannelList[Space];
}


const CChannelList *CTuningSpaceList::GetChannelList(int Space) const
{
	if (Space<0 || Space>=m_NumSpaces) {
		TRACE(TEXT("CTuningSpaceList::GetChannelList() const : Out of range %d\n"),Space);
		return NULL;
	}
	return m_ppChannelList[Space];
}


bool CTuningSpaceList::Create(const CChannelList *pList,int NumSpaces)
{
	int i;
	int Space;

	Clear();
	for (i=0;i<pList->NumChannels();i++) {
		Space=pList->GetSpace(i);
		if (Space+1>NumSpaces)
			NumSpaces=Space+1;
	}
	if (NumSpaces<1)
		return false;
	m_NumSpaces=NumSpaces;
	m_ppChannelList=new CChannelList*[m_NumSpaces];
	for (i=0;i<m_NumSpaces;i++)
		m_ppChannelList[i]=new CChannelList;
	for (i=0;i<pList->NumChannels();i++) {
		m_ppChannelList[pList->GetSpace(i)]->AddChannel(*pList->GetChannelInfo(i));
	}
	return true;
}


bool CTuningSpaceList::Reserve(int NumSpaces)
{
	if (NumSpaces<1)
		return false;
	Clear();
	m_ppChannelList=new CChannelList*[NumSpaces];
	for (int i=0;i<NumSpaces;i++)
		m_ppChannelList[i]=new CChannelList;
	m_NumSpaces=NumSpaces;
	return true;
}


void CTuningSpaceList::Clear()
{
	if (m_ppChannelList!=NULL) {
		int i;

		for (i=m_NumSpaces-1;i>=0;i--)
			delete m_ppChannelList[i];
		delete [] m_ppChannelList;
		m_ppChannelList=NULL;
		m_NumSpaces=0;
	}
}
